/****************************************************************************
** Meta object code from reading C++ file 'dnotitlebarwindowhelper.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../dnotitlebarwindowhelper.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dnotitlebarwindowhelper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_deepin_platform_plugin__DNoTitlebarWindowHelper_t {
    QByteArrayData data[39];
    char stringdata0[925];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_deepin_platform_plugin__DNoTitlebarWindowHelper_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_deepin_platform_plugin__DNoTitlebarWindowHelper_t qt_meta_stringdata_deepin_platform_plugin__DNoTitlebarWindowHelper = {
    {
QT_MOC_LITERAL(0, 0, 47), // "deepin_platform_plugin::DNoTi..."
QT_MOC_LITERAL(1, 48, 12), // "themeChanged"
QT_MOC_LITERAL(2, 61, 0), // ""
QT_MOC_LITERAL(3, 62, 19), // "windowRadiusChanged"
QT_MOC_LITERAL(4, 82, 18), // "borderWidthChanged"
QT_MOC_LITERAL(5, 101, 18), // "borderColorChanged"
QT_MOC_LITERAL(6, 120, 19), // "shadowRadiusChanged"
QT_MOC_LITERAL(7, 140, 19), // "shadowOffectChanged"
QT_MOC_LITERAL(8, 160, 18), // "shadowColorChanged"
QT_MOC_LITERAL(9, 179, 19), // "windowEffectChanged"
QT_MOC_LITERAL(10, 199, 26), // "windowStartUpEffectChanged"
QT_MOC_LITERAL(11, 226, 28), // "mouseInputAreaMarginsChanged"
QT_MOC_LITERAL(12, 255, 26), // "updateClipPathFromProperty"
QT_MOC_LITERAL(13, 282, 27), // "updateFrameMaskFromProperty"
QT_MOC_LITERAL(14, 310, 30), // "updateWindowRadiusFromProperty"
QT_MOC_LITERAL(15, 341, 29), // "updateBorderWidthFromProperty"
QT_MOC_LITERAL(16, 371, 29), // "updateBorderColorFromProperty"
QT_MOC_LITERAL(17, 401, 30), // "updateShadowRadiusFromProperty"
QT_MOC_LITERAL(18, 432, 30), // "updateShadowOffsetFromProperty"
QT_MOC_LITERAL(19, 463, 29), // "updateShadowColorFromProperty"
QT_MOC_LITERAL(20, 493, 30), // "updateWindowEffectFromProperty"
QT_MOC_LITERAL(21, 524, 37), // "updateWindowStartUpEffectFrom..."
QT_MOC_LITERAL(22, 562, 36), // "updateEnableSystemResizeFromP..."
QT_MOC_LITERAL(23, 599, 34), // "updateEnableSystemMoveFromPro..."
QT_MOC_LITERAL(24, 634, 34), // "updateEnableBlurWindowFromPro..."
QT_MOC_LITERAL(25, 669, 33), // "updateWindowBlurAreasFromProp..."
QT_MOC_LITERAL(26, 703, 33), // "updateWindowBlurPathsFromProp..."
QT_MOC_LITERAL(27, 737, 41), // "updateAutoInputMaskByClipPath..."
QT_MOC_LITERAL(28, 779, 5), // "theme"
QT_MOC_LITERAL(29, 785, 12), // "windowRadius"
QT_MOC_LITERAL(30, 798, 11), // "borderWidth"
QT_MOC_LITERAL(31, 810, 11), // "borderColor"
QT_MOC_LITERAL(32, 822, 12), // "shadowRadius"
QT_MOC_LITERAL(33, 835, 12), // "shadowOffset"
QT_MOC_LITERAL(34, 848, 11), // "shadowColor"
QT_MOC_LITERAL(35, 860, 12), // "windowEffect"
QT_MOC_LITERAL(36, 873, 19), // "windowStartUpEffect"
QT_MOC_LITERAL(37, 893, 21), // "mouseInputAreaMargins"
QT_MOC_LITERAL(38, 915, 9) // "QMarginsF"

    },
    "deepin_platform_plugin::DNoTitlebarWindowHelper\0"
    "themeChanged\0\0windowRadiusChanged\0"
    "borderWidthChanged\0borderColorChanged\0"
    "shadowRadiusChanged\0shadowOffectChanged\0"
    "shadowColorChanged\0windowEffectChanged\0"
    "windowStartUpEffectChanged\0"
    "mouseInputAreaMarginsChanged\0"
    "updateClipPathFromProperty\0"
    "updateFrameMaskFromProperty\0"
    "updateWindowRadiusFromProperty\0"
    "updateBorderWidthFromProperty\0"
    "updateBorderColorFromProperty\0"
    "updateShadowRadiusFromProperty\0"
    "updateShadowOffsetFromProperty\0"
    "updateShadowColorFromProperty\0"
    "updateWindowEffectFromProperty\0"
    "updateWindowStartUpEffectFromProperty\0"
    "updateEnableSystemResizeFromProperty\0"
    "updateEnableSystemMoveFromProperty\0"
    "updateEnableBlurWindowFromProperty\0"
    "updateWindowBlurAreasFromProperty\0"
    "updateWindowBlurPathsFromProperty\0"
    "updateAutoInputMaskByClipPathFromProperty\0"
    "theme\0windowRadius\0borderWidth\0"
    "borderColor\0shadowRadius\0shadowOffset\0"
    "shadowColor\0windowEffect\0windowStartUpEffect\0"
    "mouseInputAreaMargins\0QMarginsF"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_deepin_platform_plugin__DNoTitlebarWindowHelper[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      26,   14, // methods
      10,  170, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  144,    2, 0x06 /* Public */,
       3,    0,  145,    2, 0x06 /* Public */,
       4,    0,  146,    2, 0x06 /* Public */,
       5,    0,  147,    2, 0x06 /* Public */,
       6,    0,  148,    2, 0x06 /* Public */,
       7,    0,  149,    2, 0x06 /* Public */,
       8,    0,  150,    2, 0x06 /* Public */,
       9,    0,  151,    2, 0x06 /* Public */,
      10,    0,  152,    2, 0x06 /* Public */,
      11,    0,  153,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    0,  154,    2, 0x08 /* Private */,
      13,    0,  155,    2, 0x08 /* Private */,
      14,    0,  156,    2, 0x08 /* Private */,
      15,    0,  157,    2, 0x08 /* Private */,
      16,    0,  158,    2, 0x08 /* Private */,
      17,    0,  159,    2, 0x08 /* Private */,
      18,    0,  160,    2, 0x08 /* Private */,
      19,    0,  161,    2, 0x08 /* Private */,
      20,    0,  162,    2, 0x08 /* Private */,
      21,    0,  163,    2, 0x08 /* Private */,
      22,    0,  164,    2, 0x08 /* Private */,
      23,    0,  165,    2, 0x08 /* Private */,
      24,    0,  166,    2, 0x08 /* Private */,
      25,    0,  167,    2, 0x08 /* Private */,
      26,    0,  168,    2, 0x08 /* Private */,
      27,    0,  169,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      28, QMetaType::QString, 0x00495103,
      29, QMetaType::QPointF, 0x00495103,
      30, QMetaType::QReal, 0x00495103,
      31, QMetaType::QColor, 0x00495103,
      32, QMetaType::QReal, 0x00495103,
      33, QMetaType::QPointF, 0x00495003,
      34, QMetaType::QColor, 0x00495103,
      35, QMetaType::UInt, 0x00495103,
      36, QMetaType::UInt, 0x00495103,
      37, 0x80000000 | 38, 0x0049510b,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,

       0        // eod
};

void deepin_platform_plugin::DNoTitlebarWindowHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DNoTitlebarWindowHelper *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->themeChanged(); break;
        case 1: _t->windowRadiusChanged(); break;
        case 2: _t->borderWidthChanged(); break;
        case 3: _t->borderColorChanged(); break;
        case 4: _t->shadowRadiusChanged(); break;
        case 5: _t->shadowOffectChanged(); break;
        case 6: _t->shadowColorChanged(); break;
        case 7: _t->windowEffectChanged(); break;
        case 8: _t->windowStartUpEffectChanged(); break;
        case 9: _t->mouseInputAreaMarginsChanged(); break;
        case 10: _t->updateClipPathFromProperty(); break;
        case 11: _t->updateFrameMaskFromProperty(); break;
        case 12: _t->updateWindowRadiusFromProperty(); break;
        case 13: _t->updateBorderWidthFromProperty(); break;
        case 14: _t->updateBorderColorFromProperty(); break;
        case 15: _t->updateShadowRadiusFromProperty(); break;
        case 16: _t->updateShadowOffsetFromProperty(); break;
        case 17: _t->updateShadowColorFromProperty(); break;
        case 18: _t->updateWindowEffectFromProperty(); break;
        case 19: _t->updateWindowStartUpEffectFromProperty(); break;
        case 20: _t->updateEnableSystemResizeFromProperty(); break;
        case 21: _t->updateEnableSystemMoveFromProperty(); break;
        case 22: _t->updateEnableBlurWindowFromProperty(); break;
        case 23: _t->updateWindowBlurAreasFromProperty(); break;
        case 24: _t->updateWindowBlurPathsFromProperty(); break;
        case 25: _t->updateAutoInputMaskByClipPathFromProperty(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DNoTitlebarWindowHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DNoTitlebarWindowHelper::themeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DNoTitlebarWindowHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DNoTitlebarWindowHelper::windowRadiusChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DNoTitlebarWindowHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DNoTitlebarWindowHelper::borderWidthChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DNoTitlebarWindowHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DNoTitlebarWindowHelper::borderColorChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DNoTitlebarWindowHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DNoTitlebarWindowHelper::shadowRadiusChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DNoTitlebarWindowHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DNoTitlebarWindowHelper::shadowOffectChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DNoTitlebarWindowHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DNoTitlebarWindowHelper::shadowColorChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (DNoTitlebarWindowHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DNoTitlebarWindowHelper::windowEffectChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (DNoTitlebarWindowHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DNoTitlebarWindowHelper::windowStartUpEffectChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (DNoTitlebarWindowHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DNoTitlebarWindowHelper::mouseInputAreaMarginsChanged)) {
                *result = 9;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 9:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMarginsF >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DNoTitlebarWindowHelper *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->theme(); break;
        case 1: *reinterpret_cast< QPointF*>(_v) = _t->windowRadius(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->borderWidth(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->borderColor(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->shadowRadius(); break;
        case 5: *reinterpret_cast< QPointF*>(_v) = _t->shadowOffset(); break;
        case 6: *reinterpret_cast< QColor*>(_v) = _t->shadowColor(); break;
        case 7: *reinterpret_cast< quint32*>(_v) = _t->windowEffect(); break;
        case 8: *reinterpret_cast< quint32*>(_v) = _t->windowStartUpEffect(); break;
        case 9: *reinterpret_cast< QMarginsF*>(_v) = _t->mouseInputAreaMargins(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DNoTitlebarWindowHelper *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTheme(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setWindowRadius(*reinterpret_cast< QPointF*>(_v)); break;
        case 2: _t->setBorderWidth(*reinterpret_cast< qreal*>(_v)); break;
        case 3: _t->setBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setShadowRadius(*reinterpret_cast< qreal*>(_v)); break;
        case 5: _t->setShadowOffect(*reinterpret_cast< QPointF*>(_v)); break;
        case 6: _t->setShadowColor(*reinterpret_cast< QColor*>(_v)); break;
        case 7: _t->setWindowEffect(*reinterpret_cast< quint32*>(_v)); break;
        case 8: _t->setWindowStartUpEffect(*reinterpret_cast< quint32*>(_v)); break;
        case 9: _t->setMouseInputAreaMargins(*reinterpret_cast< QMarginsF*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject deepin_platform_plugin::DNoTitlebarWindowHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_deepin_platform_plugin__DNoTitlebarWindowHelper.data,
    qt_meta_data_deepin_platform_plugin__DNoTitlebarWindowHelper,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *deepin_platform_plugin::DNoTitlebarWindowHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *deepin_platform_plugin::DNoTitlebarWindowHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_deepin_platform_plugin__DNoTitlebarWindowHelper.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int deepin_platform_plugin::DNoTitlebarWindowHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 26;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 10;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void deepin_platform_plugin::DNoTitlebarWindowHelper::themeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void deepin_platform_plugin::DNoTitlebarWindowHelper::windowRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void deepin_platform_plugin::DNoTitlebarWindowHelper::borderWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void deepin_platform_plugin::DNoTitlebarWindowHelper::borderColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void deepin_platform_plugin::DNoTitlebarWindowHelper::shadowRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void deepin_platform_plugin::DNoTitlebarWindowHelper::shadowOffectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void deepin_platform_plugin::DNoTitlebarWindowHelper::shadowColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void deepin_platform_plugin::DNoTitlebarWindowHelper::windowEffectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void deepin_platform_plugin::DNoTitlebarWindowHelper::windowStartUpEffectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void deepin_platform_plugin::DNoTitlebarWindowHelper::mouseInputAreaMarginsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
