/****************************************************************************
** Meta object code from reading C++ file 'dxcbwmsupport.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../dxcbwmsupport.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dxcbwmsupport.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22deepin_platform_plugin13DXcbWMSupportE_t {};
} // unnamed namespace

template <> constexpr inline auto deepin_platform_plugin::DXcbWMSupport::qt_create_metaobjectdata<qt_meta_tag_ZN22deepin_platform_plugin13DXcbWMSupportE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "deepin_platform_plugin::DXcbWMSupport",
        "windowManagerChanged",
        "",
        "hasBlurWindowChanged",
        "hasBlurWindow",
        "hasCompositeChanged",
        "hasComposite",
        "hasNoTitlebarChanged",
        "hasNoTitlebar",
        "hasScissorWindowChanged",
        "hasScissorWindow",
        "hasWallpaperEffectChanged",
        "hasWallpaperEffect",
        "windowListChanged",
        "windowMotifWMHintsChanged",
        "winId",
        "wallpaperSharedChanged",
        "hasWindowAlpha"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'windowManagerChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasBlurWindowChanged'
        QtMocHelpers::SignalData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Signal 'hasCompositeChanged'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Signal 'hasNoTitlebarChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Signal 'hasScissorWindowChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'hasWallpaperEffectChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Signal 'windowListChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowMotifWMHintsChanged'
        QtMocHelpers::SignalData<void(quint32)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 15 },
        }}),
        // Signal 'wallpaperSharedChanged'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'hasBlurWindow'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'hasComposite'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'hasWindowAlpha'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final),
        // property 'hasNoTitlebar'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'hasScissorWindow'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
        // property 'hasWallpaperEffect'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DXcbWMSupport, qt_meta_tag_ZN22deepin_platform_plugin13DXcbWMSupportE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject deepin_platform_plugin::DXcbWMSupport::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22deepin_platform_plugin13DXcbWMSupportE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22deepin_platform_plugin13DXcbWMSupportE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22deepin_platform_plugin13DXcbWMSupportE_t>.metaTypes,
    nullptr
} };

void deepin_platform_plugin::DXcbWMSupport::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DXcbWMSupport *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->windowManagerChanged(); break;
        case 1: _t->hasBlurWindowChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->hasCompositeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->hasNoTitlebarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->hasScissorWindowChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->hasWallpaperEffectChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->windowListChanged(); break;
        case 7: _t->windowMotifWMHintsChanged((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1]))); break;
        case 8: _t->wallpaperSharedChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DXcbWMSupport::*)()>(_a, &DXcbWMSupport::windowManagerChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DXcbWMSupport::*)(bool )>(_a, &DXcbWMSupport::hasBlurWindowChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DXcbWMSupport::*)(bool )>(_a, &DXcbWMSupport::hasCompositeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DXcbWMSupport::*)(bool )>(_a, &DXcbWMSupport::hasNoTitlebarChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DXcbWMSupport::*)(bool )>(_a, &DXcbWMSupport::hasScissorWindowChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DXcbWMSupport::*)(bool )>(_a, &DXcbWMSupport::hasWallpaperEffectChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DXcbWMSupport::*)()>(_a, &DXcbWMSupport::windowListChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DXcbWMSupport::*)(quint32 )>(_a, &DXcbWMSupport::windowMotifWMHintsChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DXcbWMSupport::*)()>(_a, &DXcbWMSupport::wallpaperSharedChanged, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->hasBlurWindow(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->hasComposite(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->hasWindowAlpha(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->hasNoTitlebar(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->hasScissorWindow(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->hasWallpaperEffect(); break;
        default: break;
        }
    }
}

const QMetaObject *deepin_platform_plugin::DXcbWMSupport::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *deepin_platform_plugin::DXcbWMSupport::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22deepin_platform_plugin13DXcbWMSupportE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int deepin_platform_plugin::DXcbWMSupport::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void deepin_platform_plugin::DXcbWMSupport::windowManagerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void deepin_platform_plugin::DXcbWMSupport::hasBlurWindowChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void deepin_platform_plugin::DXcbWMSupport::hasCompositeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void deepin_platform_plugin::DXcbWMSupport::hasNoTitlebarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void deepin_platform_plugin::DXcbWMSupport::hasScissorWindowChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void deepin_platform_plugin::DXcbWMSupport::hasWallpaperEffectChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void deepin_platform_plugin::DXcbWMSupport::windowListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void deepin_platform_plugin::DXcbWMSupport::windowMotifWMHintsChanged(quint32 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void deepin_platform_plugin::DXcbWMSupport::wallpaperSharedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
