/****************************************************************************
** Meta object code from reading C++ file 'dbus_interface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "dbus_interface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbus_interface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN27OrgFreedesktopDBusInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgFreedesktopDBusInterface::qt_create_metaobjectdata<qt_meta_tag_ZN27OrgFreedesktopDBusInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgFreedesktopDBusInterface",
        "NameAcquired",
        "",
        "in0",
        "NameLost",
        "NameOwnerChanged",
        "in1",
        "in2",
        "AddMatch",
        "QDBusPendingReply<>",
        "GetAdtAuditSessionData",
        "QDBusPendingReply<QByteArray>",
        "GetConnectionSELinuxSecurityContext",
        "GetConnectionUnixProcessID",
        "QDBusPendingReply<uint>",
        "GetConnectionUnixUser",
        "GetId",
        "QDBusPendingReply<QString>",
        "GetNameOwner",
        "Hello",
        "ListActivatableNames",
        "QDBusPendingReply<QStringList>",
        "ListNames",
        "ListQueuedOwners",
        "NameHasOwner",
        "QDBusPendingReply<bool>",
        "ReleaseName",
        "ReloadConfig",
        "RemoveMatch",
        "RequestName",
        "StartServiceByName"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'NameAcquired'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'NameLost'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'NameOwnerChanged'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 6 }, { QMetaType::QString, 7 },
        }}),
        // Slot 'AddMatch'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(8, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'GetAdtAuditSessionData'
        QtMocHelpers::SlotData<QDBusPendingReply<QByteArray>(const QString &)>(10, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'GetConnectionSELinuxSecurityContext'
        QtMocHelpers::SlotData<QDBusPendingReply<QByteArray>(const QString &)>(12, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'GetConnectionUnixProcessID'
        QtMocHelpers::SlotData<QDBusPendingReply<uint>(const QString &)>(13, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'GetConnectionUnixUser'
        QtMocHelpers::SlotData<QDBusPendingReply<uint>(const QString &)>(15, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'GetId'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(16, 2, QMC::AccessPublic, 0x80000000 | 17),
        // Slot 'GetNameOwner'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(const QString &)>(18, 2, QMC::AccessPublic, 0x80000000 | 17, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'Hello'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(19, 2, QMC::AccessPublic, 0x80000000 | 17),
        // Slot 'ListActivatableNames'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>()>(20, 2, QMC::AccessPublic, 0x80000000 | 21),
        // Slot 'ListNames'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>()>(22, 2, QMC::AccessPublic, 0x80000000 | 21),
        // Slot 'ListQueuedOwners'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>(const QString &)>(23, 2, QMC::AccessPublic, 0x80000000 | 21, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'NameHasOwner'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &)>(24, 2, QMC::AccessPublic, 0x80000000 | 25, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'ReleaseName'
        QtMocHelpers::SlotData<QDBusPendingReply<uint>(const QString &)>(26, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'ReloadConfig'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(27, 2, QMC::AccessPublic, 0x80000000 | 9),
        // Slot 'RemoveMatch'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(28, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'RequestName'
        QtMocHelpers::SlotData<QDBusPendingReply<uint>(const QString &, uint)>(29, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 }, { QMetaType::UInt, 6 },
        }}),
        // Slot 'StartServiceByName'
        QtMocHelpers::SlotData<QDBusPendingReply<uint>(const QString &, uint)>(30, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 }, { QMetaType::UInt, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgFreedesktopDBusInterface, qt_meta_tag_ZN27OrgFreedesktopDBusInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgFreedesktopDBusInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27OrgFreedesktopDBusInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27OrgFreedesktopDBusInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27OrgFreedesktopDBusInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgFreedesktopDBusInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopDBusInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->NameAcquired((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->NameLost((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->NameOwnerChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 3: { QDBusPendingReply<> _r = _t->AddMatch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<QByteArray> _r = _t->GetAdtAuditSessionData((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QByteArray>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<QByteArray> _r = _t->GetConnectionSELinuxSecurityContext((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QByteArray>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<uint> _r = _t->GetConnectionUnixProcessID((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<uint>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<uint> _r = _t->GetConnectionUnixUser((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<uint>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<QString> _r = _t->GetId();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<QString> _r = _t->GetNameOwner((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<QString> _r = _t->Hello();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<QStringList> _r = _t->ListActivatableNames();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<QStringList> _r = _t->ListNames();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 13: { QDBusPendingReply<QStringList> _r = _t->ListQueuedOwners((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 14: { QDBusPendingReply<bool> _r = _t->NameHasOwner((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 15: { QDBusPendingReply<uint> _r = _t->ReleaseName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<uint>*>(_a[0]) = std::move(_r); }  break;
        case 16: { QDBusPendingReply<> _r = _t->ReloadConfig();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 17: { QDBusPendingReply<> _r = _t->RemoveMatch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 18: { QDBusPendingReply<uint> _r = _t->RequestName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<uint>*>(_a[0]) = std::move(_r); }  break;
        case 19: { QDBusPendingReply<uint> _r = _t->StartServiceByName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<uint>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopDBusInterface::*)(const QString & )>(_a, &OrgFreedesktopDBusInterface::NameAcquired, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopDBusInterface::*)(const QString & )>(_a, &OrgFreedesktopDBusInterface::NameLost, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopDBusInterface::*)(const QString & , const QString & , const QString & )>(_a, &OrgFreedesktopDBusInterface::NameOwnerChanged, 2))
            return;
    }
}

const QMetaObject *OrgFreedesktopDBusInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgFreedesktopDBusInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27OrgFreedesktopDBusInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgFreedesktopDBusInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void OrgFreedesktopDBusInterface::NameAcquired(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OrgFreedesktopDBusInterface::NameLost(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OrgFreedesktopDBusInterface::NameOwnerChanged(const QString & _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
