/****************************************************************************
** Meta object code from reading C++ file 'dplatformwindowhelper.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../dplatformwindowhelper.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dplatformwindowhelper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_deepin_platform_plugin__DPlatformWindowHelper_t {
    QByteArrayData data[16];
    char stringdata0[502];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_deepin_platform_plugin__DPlatformWindowHelper_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_deepin_platform_plugin__DPlatformWindowHelper_t qt_meta_stringdata_deepin_platform_plugin__DPlatformWindowHelper = {
    {
QT_MOC_LITERAL(0, 0, 45), // "deepin_platform_plugin::DPlat..."
QT_MOC_LITERAL(1, 46, 26), // "updateClipPathFromProperty"
QT_MOC_LITERAL(2, 73, 0), // ""
QT_MOC_LITERAL(3, 74, 27), // "updateFrameMaskFromProperty"
QT_MOC_LITERAL(4, 102, 30), // "updateWindowRadiusFromProperty"
QT_MOC_LITERAL(5, 133, 29), // "updateBorderWidthFromProperty"
QT_MOC_LITERAL(6, 163, 29), // "updateBorderColorFromProperty"
QT_MOC_LITERAL(7, 193, 30), // "updateShadowRadiusFromProperty"
QT_MOC_LITERAL(8, 224, 30), // "updateShadowOffsetFromProperty"
QT_MOC_LITERAL(9, 255, 29), // "updateShadowColorFromProperty"
QT_MOC_LITERAL(10, 285, 36), // "updateEnableSystemResizeFromP..."
QT_MOC_LITERAL(11, 322, 34), // "updateEnableSystemMoveFromPro..."
QT_MOC_LITERAL(12, 357, 34), // "updateEnableBlurWindowFromPro..."
QT_MOC_LITERAL(13, 392, 33), // "updateWindowBlurAreasFromProp..."
QT_MOC_LITERAL(14, 426, 33), // "updateWindowBlurPathsFromProp..."
QT_MOC_LITERAL(15, 460, 41) // "updateAutoInputMaskByClipPath..."

    },
    "deepin_platform_plugin::DPlatformWindowHelper\0"
    "updateClipPathFromProperty\0\0"
    "updateFrameMaskFromProperty\0"
    "updateWindowRadiusFromProperty\0"
    "updateBorderWidthFromProperty\0"
    "updateBorderColorFromProperty\0"
    "updateShadowRadiusFromProperty\0"
    "updateShadowOffsetFromProperty\0"
    "updateShadowColorFromProperty\0"
    "updateEnableSystemResizeFromProperty\0"
    "updateEnableSystemMoveFromProperty\0"
    "updateEnableBlurWindowFromProperty\0"
    "updateWindowBlurAreasFromProperty\0"
    "updateWindowBlurPathsFromProperty\0"
    "updateAutoInputMaskByClipPathFromProperty"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_deepin_platform_plugin__DPlatformWindowHelper[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   84,    2, 0x08 /* Private */,
       3,    0,   85,    2, 0x08 /* Private */,
       4,    0,   86,    2, 0x08 /* Private */,
       5,    0,   87,    2, 0x08 /* Private */,
       6,    0,   88,    2, 0x08 /* Private */,
       7,    0,   89,    2, 0x08 /* Private */,
       8,    0,   90,    2, 0x08 /* Private */,
       9,    0,   91,    2, 0x08 /* Private */,
      10,    0,   92,    2, 0x08 /* Private */,
      11,    0,   93,    2, 0x08 /* Private */,
      12,    0,   94,    2, 0x08 /* Private */,
      13,    0,   95,    2, 0x08 /* Private */,
      14,    0,   96,    2, 0x08 /* Private */,
      15,    0,   97,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void deepin_platform_plugin::DPlatformWindowHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DPlatformWindowHelper *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateClipPathFromProperty(); break;
        case 1: _t->updateFrameMaskFromProperty(); break;
        case 2: _t->updateWindowRadiusFromProperty(); break;
        case 3: _t->updateBorderWidthFromProperty(); break;
        case 4: _t->updateBorderColorFromProperty(); break;
        case 5: _t->updateShadowRadiusFromProperty(); break;
        case 6: _t->updateShadowOffsetFromProperty(); break;
        case 7: _t->updateShadowColorFromProperty(); break;
        case 8: _t->updateEnableSystemResizeFromProperty(); break;
        case 9: _t->updateEnableSystemMoveFromProperty(); break;
        case 10: _t->updateEnableBlurWindowFromProperty(); break;
        case 11: _t->updateWindowBlurAreasFromProperty(); break;
        case 12: _t->updateWindowBlurPathsFromProperty(); break;
        case 13: _t->updateAutoInputMaskByClipPathFromProperty(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject deepin_platform_plugin::DPlatformWindowHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_deepin_platform_plugin__DPlatformWindowHelper.data,
    qt_meta_data_deepin_platform_plugin__DPlatformWindowHelper,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *deepin_platform_plugin::DPlatformWindowHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *deepin_platform_plugin::DPlatformWindowHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_deepin_platform_plugin__DPlatformWindowHelper.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int deepin_platform_plugin::DPlatformWindowHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
