/****************************************************************************
** Meta object code from reading C++ file 'ddesktopinputselectioncontrol.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/ddesktopinputselectioncontrol.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ddesktopinputselectioncontrol.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_deepin_platform_plugin__DDesktopInputSelectionControl_t {
    QByteArrayData data[17];
    char stringdata0[363];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_deepin_platform_plugin__DDesktopInputSelectionControl_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_deepin_platform_plugin__DDesktopInputSelectionControl_t qt_meta_stringdata_deepin_platform_plugin__DDesktopInputSelectionControl = {
    {
QT_MOC_LITERAL(0, 0, 53), // "deepin_platform_plugin::DDesk..."
QT_MOC_LITERAL(1, 54, 21), // "anchorPositionChanged"
QT_MOC_LITERAL(2, 76, 0), // ""
QT_MOC_LITERAL(3, 77, 21), // "cursorPositionChanged"
QT_MOC_LITERAL(4, 99, 22), // "anchorRectangleChanged"
QT_MOC_LITERAL(5, 122, 22), // "cursorRectangleChanged"
QT_MOC_LITERAL(6, 145, 30), // "selectionControlVisibleChanged"
QT_MOC_LITERAL(7, 176, 26), // "updateAnchorHandlePosition"
QT_MOC_LITERAL(8, 203, 26), // "updateCursorHandlePosition"
QT_MOC_LITERAL(9, 230, 21), // "updateTooltipPosition"
QT_MOC_LITERAL(10, 252, 20), // "onWindowStateChanged"
QT_MOC_LITERAL(11, 273, 15), // "Qt::WindowState"
QT_MOC_LITERAL(12, 289, 5), // "state"
QT_MOC_LITERAL(13, 295, 29), // "updateSelectionControlVisible"
QT_MOC_LITERAL(14, 325, 11), // "onOptAction"
QT_MOC_LITERAL(15, 337, 4), // "type"
QT_MOC_LITERAL(16, 342, 20) // "onFocusWindowChanged"

    },
    "deepin_platform_plugin::DDesktopInputSelectionControl\0"
    "anchorPositionChanged\0\0cursorPositionChanged\0"
    "anchorRectangleChanged\0cursorRectangleChanged\0"
    "selectionControlVisibleChanged\0"
    "updateAnchorHandlePosition\0"
    "updateCursorHandlePosition\0"
    "updateTooltipPosition\0onWindowStateChanged\0"
    "Qt::WindowState\0state\0"
    "updateSelectionControlVisible\0onOptAction\0"
    "type\0onFocusWindowChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_deepin_platform_plugin__DDesktopInputSelectionControl[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x06 /* Public */,
       3,    0,   75,    2, 0x06 /* Public */,
       4,    0,   76,    2, 0x06 /* Public */,
       5,    0,   77,    2, 0x06 /* Public */,
       6,    0,   78,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,   79,    2, 0x0a /* Public */,
       8,    0,   80,    2, 0x0a /* Public */,
       9,    0,   81,    2, 0x0a /* Public */,
      10,    1,   82,    2, 0x0a /* Public */,
      13,    0,   85,    2, 0x0a /* Public */,
      14,    1,   86,    2, 0x0a /* Public */,
      16,    0,   89,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void,

       0        // eod
};

void deepin_platform_plugin::DDesktopInputSelectionControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DDesktopInputSelectionControl *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->anchorPositionChanged(); break;
        case 1: _t->cursorPositionChanged(); break;
        case 2: _t->anchorRectangleChanged(); break;
        case 3: _t->cursorRectangleChanged(); break;
        case 4: _t->selectionControlVisibleChanged(); break;
        case 5: _t->updateAnchorHandlePosition(); break;
        case 6: _t->updateCursorHandlePosition(); break;
        case 7: _t->updateTooltipPosition(); break;
        case 8: _t->onWindowStateChanged((*reinterpret_cast< Qt::WindowState(*)>(_a[1]))); break;
        case 9: _t->updateSelectionControlVisible(); break;
        case 10: _t->onOptAction((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->onFocusWindowChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DDesktopInputSelectionControl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDesktopInputSelectionControl::anchorPositionChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DDesktopInputSelectionControl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDesktopInputSelectionControl::cursorPositionChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DDesktopInputSelectionControl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDesktopInputSelectionControl::anchorRectangleChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DDesktopInputSelectionControl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDesktopInputSelectionControl::cursorRectangleChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DDesktopInputSelectionControl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDesktopInputSelectionControl::selectionControlVisibleChanged)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject deepin_platform_plugin::DDesktopInputSelectionControl::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_deepin_platform_plugin__DDesktopInputSelectionControl.data,
    qt_meta_data_deepin_platform_plugin__DDesktopInputSelectionControl,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *deepin_platform_plugin::DDesktopInputSelectionControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *deepin_platform_plugin::DDesktopInputSelectionControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_deepin_platform_plugin__DDesktopInputSelectionControl.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int deepin_platform_plugin::DDesktopInputSelectionControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void deepin_platform_plugin::DDesktopInputSelectionControl::anchorPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void deepin_platform_plugin::DDesktopInputSelectionControl::cursorPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void deepin_platform_plugin::DDesktopInputSelectionControl::anchorRectangleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void deepin_platform_plugin::DDesktopInputSelectionControl::cursorRectangleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void deepin_platform_plugin::DDesktopInputSelectionControl::selectionControlVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
