/****************************************************************************
** Meta object code from reading C++ file 'dxcbwmsupport.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../dxcbwmsupport.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dxcbwmsupport.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_deepin_platform_plugin__DXcbWMSupport_t {
    QByteArrayData data[18];
    char stringdata0[337];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_deepin_platform_plugin__DXcbWMSupport_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_deepin_platform_plugin__DXcbWMSupport_t qt_meta_stringdata_deepin_platform_plugin__DXcbWMSupport = {
    {
QT_MOC_LITERAL(0, 0, 37), // "deepin_platform_plugin::DXcbW..."
QT_MOC_LITERAL(1, 38, 20), // "windowManagerChanged"
QT_MOC_LITERAL(2, 59, 0), // ""
QT_MOC_LITERAL(3, 60, 20), // "hasBlurWindowChanged"
QT_MOC_LITERAL(4, 81, 13), // "hasBlurWindow"
QT_MOC_LITERAL(5, 95, 19), // "hasCompositeChanged"
QT_MOC_LITERAL(6, 115, 12), // "hasComposite"
QT_MOC_LITERAL(7, 128, 20), // "hasNoTitlebarChanged"
QT_MOC_LITERAL(8, 149, 13), // "hasNoTitlebar"
QT_MOC_LITERAL(9, 163, 23), // "hasScissorWindowChanged"
QT_MOC_LITERAL(10, 187, 16), // "hasScissorWindow"
QT_MOC_LITERAL(11, 204, 25), // "hasWallpaperEffectChanged"
QT_MOC_LITERAL(12, 230, 18), // "hasWallpaperEffect"
QT_MOC_LITERAL(13, 249, 17), // "windowListChanged"
QT_MOC_LITERAL(14, 267, 25), // "windowMotifWMHintsChanged"
QT_MOC_LITERAL(15, 293, 5), // "winId"
QT_MOC_LITERAL(16, 299, 22), // "wallpaperSharedChanged"
QT_MOC_LITERAL(17, 322, 14) // "hasWindowAlpha"

    },
    "deepin_platform_plugin::DXcbWMSupport\0"
    "windowManagerChanged\0\0hasBlurWindowChanged\0"
    "hasBlurWindow\0hasCompositeChanged\0"
    "hasComposite\0hasNoTitlebarChanged\0"
    "hasNoTitlebar\0hasScissorWindowChanged\0"
    "hasScissorWindow\0hasWallpaperEffectChanged\0"
    "hasWallpaperEffect\0windowListChanged\0"
    "windowMotifWMHintsChanged\0winId\0"
    "wallpaperSharedChanged\0hasWindowAlpha"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_deepin_platform_plugin__DXcbWMSupport[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       6,   80, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x06 /* Public */,
       3,    1,   60,    2, 0x06 /* Public */,
       5,    1,   63,    2, 0x06 /* Public */,
       7,    1,   66,    2, 0x06 /* Public */,
       9,    1,   69,    2, 0x06 /* Public */,
      11,    1,   72,    2, 0x06 /* Public */,
      13,    0,   75,    2, 0x06 /* Public */,
      14,    1,   76,    2, 0x06 /* Public */,
      16,    0,   79,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    4,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void,
    QMetaType::Void, QMetaType::UInt,   15,
    QMetaType::Void,

 // properties: name, type, flags
       4, QMetaType::Bool, 0x00495001,
       6, QMetaType::Bool, 0x00495001,
      17, QMetaType::Bool, 0x00095801,
       8, QMetaType::Bool, 0x00495001,
      10, QMetaType::Bool, 0x00495001,
      12, QMetaType::Bool, 0x00495001,

 // properties: notify_signal_id
       1,
       2,
       0,
       3,
       4,
       5,

       0        // eod
};

void deepin_platform_plugin::DXcbWMSupport::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DXcbWMSupport *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->windowManagerChanged(); break;
        case 1: _t->hasBlurWindowChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->hasCompositeChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->hasNoTitlebarChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->hasScissorWindowChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->hasWallpaperEffectChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->windowListChanged(); break;
        case 7: _t->windowMotifWMHintsChanged((*reinterpret_cast< quint32(*)>(_a[1]))); break;
        case 8: _t->wallpaperSharedChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DXcbWMSupport::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DXcbWMSupport::windowManagerChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DXcbWMSupport::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DXcbWMSupport::hasBlurWindowChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DXcbWMSupport::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DXcbWMSupport::hasCompositeChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DXcbWMSupport::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DXcbWMSupport::hasNoTitlebarChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DXcbWMSupport::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DXcbWMSupport::hasScissorWindowChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DXcbWMSupport::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DXcbWMSupport::hasWallpaperEffectChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DXcbWMSupport::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DXcbWMSupport::windowListChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (DXcbWMSupport::*)(quint32 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DXcbWMSupport::windowMotifWMHintsChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (DXcbWMSupport::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DXcbWMSupport::wallpaperSharedChanged)) {
                *result = 8;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DXcbWMSupport *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->hasBlurWindow(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->hasComposite(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->hasWindowAlpha(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->hasNoTitlebar(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->hasScissorWindow(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->hasWallpaperEffect(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject deepin_platform_plugin::DXcbWMSupport::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_deepin_platform_plugin__DXcbWMSupport.data,
    qt_meta_data_deepin_platform_plugin__DXcbWMSupport,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *deepin_platform_plugin::DXcbWMSupport::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *deepin_platform_plugin::DXcbWMSupport::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_deepin_platform_plugin__DXcbWMSupport.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int deepin_platform_plugin::DXcbWMSupport::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 6;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void deepin_platform_plugin::DXcbWMSupport::windowManagerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void deepin_platform_plugin::DXcbWMSupport::hasBlurWindowChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void deepin_platform_plugin::DXcbWMSupport::hasCompositeChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void deepin_platform_plugin::DXcbWMSupport::hasNoTitlebarChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void deepin_platform_plugin::DXcbWMSupport::hasScissorWindowChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void deepin_platform_plugin::DXcbWMSupport::hasWallpaperEffectChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void deepin_platform_plugin::DXcbWMSupport::windowListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void deepin_platform_plugin::DXcbWMSupport::windowMotifWMHintsChanged(quint32 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void deepin_platform_plugin::DXcbWMSupport::wallpaperSharedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
