/****************************************************************************
** Meta object code from reading C++ file 'dnotitlebarwindowhelper.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../dnotitlebarwindowhelper.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dnotitlebarwindowhelper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22deepin_platform_plugin23DNoTitlebarWindowHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto deepin_platform_plugin::DNoTitlebarWindowHelper::qt_create_metaobjectdata<qt_meta_tag_ZN22deepin_platform_plugin23DNoTitlebarWindowHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "deepin_platform_plugin::DNoTitlebarWindowHelper",
        "themeChanged",
        "",
        "windowRadiusChanged",
        "borderWidthChanged",
        "borderColorChanged",
        "shadowRadiusChanged",
        "shadowOffectChanged",
        "shadowColorChanged",
        "windowEffectChanged",
        "windowStartUpEffectChanged",
        "mouseInputAreaMarginsChanged",
        "updateClipPathFromProperty",
        "updateFrameMaskFromProperty",
        "updateWindowRadiusFromProperty",
        "updateBorderWidthFromProperty",
        "updateBorderColorFromProperty",
        "updateShadowRadiusFromProperty",
        "updateShadowOffsetFromProperty",
        "updateShadowColorFromProperty",
        "updateWindowEffectFromProperty",
        "updateWindowStartUpEffectFromProperty",
        "updateEnableSystemResizeFromProperty",
        "updateEnableSystemMoveFromProperty",
        "updateEnableBlurWindowFromProperty",
        "updateWindowBlurAreasFromProperty",
        "updateWindowBlurPathsFromProperty",
        "updateAutoInputMaskByClipPathFromProperty",
        "theme",
        "windowRadius",
        "QPointF",
        "borderWidth",
        "borderColor",
        "QColor",
        "shadowRadius",
        "shadowOffset",
        "shadowColor",
        "windowEffect",
        "windowStartUpEffect",
        "mouseInputAreaMargins",
        "QMarginsF"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'themeChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowRadiusChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'borderWidthChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'borderColorChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowRadiusChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowOffectChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowColorChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowEffectChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowStartUpEffectChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'mouseInputAreaMarginsChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateClipPathFromProperty'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateFrameMaskFromProperty'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateWindowRadiusFromProperty'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateBorderWidthFromProperty'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateBorderColorFromProperty'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateShadowRadiusFromProperty'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateShadowOffsetFromProperty'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateShadowColorFromProperty'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateWindowEffectFromProperty'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateWindowStartUpEffectFromProperty'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateEnableSystemResizeFromProperty'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateEnableSystemMoveFromProperty'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateEnableBlurWindowFromProperty'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateWindowBlurAreasFromProperty'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateWindowBlurPathsFromProperty'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateAutoInputMaskByClipPathFromProperty'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'theme'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'windowRadius'
        QtMocHelpers::PropertyData<QPointF>(29, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'borderWidth'
        QtMocHelpers::PropertyData<qreal>(31, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'borderColor'
        QtMocHelpers::PropertyData<QColor>(32, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'shadowRadius'
        QtMocHelpers::PropertyData<qreal>(34, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'shadowOffset'
        QtMocHelpers::PropertyData<QPointF>(35, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 5),
        // property 'shadowColor'
        QtMocHelpers::PropertyData<QColor>(36, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'windowEffect'
        QtMocHelpers::PropertyData<quint32>(37, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'windowStartUpEffect'
        QtMocHelpers::PropertyData<quint32>(38, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'mouseInputAreaMargins'
        QtMocHelpers::PropertyData<QMarginsF>(39, 0x80000000 | 40, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 9),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DNoTitlebarWindowHelper, qt_meta_tag_ZN22deepin_platform_plugin23DNoTitlebarWindowHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject deepin_platform_plugin::DNoTitlebarWindowHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22deepin_platform_plugin23DNoTitlebarWindowHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22deepin_platform_plugin23DNoTitlebarWindowHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22deepin_platform_plugin23DNoTitlebarWindowHelperE_t>.metaTypes,
    nullptr
} };

void deepin_platform_plugin::DNoTitlebarWindowHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DNoTitlebarWindowHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->themeChanged(); break;
        case 1: _t->windowRadiusChanged(); break;
        case 2: _t->borderWidthChanged(); break;
        case 3: _t->borderColorChanged(); break;
        case 4: _t->shadowRadiusChanged(); break;
        case 5: _t->shadowOffectChanged(); break;
        case 6: _t->shadowColorChanged(); break;
        case 7: _t->windowEffectChanged(); break;
        case 8: _t->windowStartUpEffectChanged(); break;
        case 9: _t->mouseInputAreaMarginsChanged(); break;
        case 10: _t->updateClipPathFromProperty(); break;
        case 11: _t->updateFrameMaskFromProperty(); break;
        case 12: _t->updateWindowRadiusFromProperty(); break;
        case 13: _t->updateBorderWidthFromProperty(); break;
        case 14: _t->updateBorderColorFromProperty(); break;
        case 15: _t->updateShadowRadiusFromProperty(); break;
        case 16: _t->updateShadowOffsetFromProperty(); break;
        case 17: _t->updateShadowColorFromProperty(); break;
        case 18: _t->updateWindowEffectFromProperty(); break;
        case 19: _t->updateWindowStartUpEffectFromProperty(); break;
        case 20: _t->updateEnableSystemResizeFromProperty(); break;
        case 21: _t->updateEnableSystemMoveFromProperty(); break;
        case 22: _t->updateEnableBlurWindowFromProperty(); break;
        case 23: _t->updateWindowBlurAreasFromProperty(); break;
        case 24: _t->updateWindowBlurPathsFromProperty(); break;
        case 25: _t->updateAutoInputMaskByClipPathFromProperty(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DNoTitlebarWindowHelper::*)()>(_a, &DNoTitlebarWindowHelper::themeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DNoTitlebarWindowHelper::*)()>(_a, &DNoTitlebarWindowHelper::windowRadiusChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DNoTitlebarWindowHelper::*)()>(_a, &DNoTitlebarWindowHelper::borderWidthChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DNoTitlebarWindowHelper::*)()>(_a, &DNoTitlebarWindowHelper::borderColorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DNoTitlebarWindowHelper::*)()>(_a, &DNoTitlebarWindowHelper::shadowRadiusChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DNoTitlebarWindowHelper::*)()>(_a, &DNoTitlebarWindowHelper::shadowOffectChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DNoTitlebarWindowHelper::*)()>(_a, &DNoTitlebarWindowHelper::shadowColorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DNoTitlebarWindowHelper::*)()>(_a, &DNoTitlebarWindowHelper::windowEffectChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DNoTitlebarWindowHelper::*)()>(_a, &DNoTitlebarWindowHelper::windowStartUpEffectChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (DNoTitlebarWindowHelper::*)()>(_a, &DNoTitlebarWindowHelper::mouseInputAreaMarginsChanged, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 9:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMarginsF >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->theme(); break;
        case 1: *reinterpret_cast<QPointF*>(_v) = _t->windowRadius(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->borderWidth(); break;
        case 3: *reinterpret_cast<QColor*>(_v) = _t->borderColor(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->shadowRadius(); break;
        case 5: *reinterpret_cast<QPointF*>(_v) = _t->shadowOffset(); break;
        case 6: *reinterpret_cast<QColor*>(_v) = _t->shadowColor(); break;
        case 7: *reinterpret_cast<quint32*>(_v) = _t->windowEffect(); break;
        case 8: *reinterpret_cast<quint32*>(_v) = _t->windowStartUpEffect(); break;
        case 9: *reinterpret_cast<QMarginsF*>(_v) = _t->mouseInputAreaMargins(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTheme(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setWindowRadius(*reinterpret_cast<QPointF*>(_v)); break;
        case 2: _t->setBorderWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setBorderColor(*reinterpret_cast<QColor*>(_v)); break;
        case 4: _t->setShadowRadius(*reinterpret_cast<qreal*>(_v)); break;
        case 5: _t->setShadowOffect(*reinterpret_cast<QPointF*>(_v)); break;
        case 6: _t->setShadowColor(*reinterpret_cast<QColor*>(_v)); break;
        case 7: _t->setWindowEffect(*reinterpret_cast<quint32*>(_v)); break;
        case 8: _t->setWindowStartUpEffect(*reinterpret_cast<quint32*>(_v)); break;
        case 9: _t->setMouseInputAreaMargins(*reinterpret_cast<QMarginsF*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *deepin_platform_plugin::DNoTitlebarWindowHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *deepin_platform_plugin::DNoTitlebarWindowHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22deepin_platform_plugin23DNoTitlebarWindowHelperE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int deepin_platform_plugin::DNoTitlebarWindowHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 26;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void deepin_platform_plugin::DNoTitlebarWindowHelper::themeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void deepin_platform_plugin::DNoTitlebarWindowHelper::windowRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void deepin_platform_plugin::DNoTitlebarWindowHelper::borderWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void deepin_platform_plugin::DNoTitlebarWindowHelper::borderColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void deepin_platform_plugin::DNoTitlebarWindowHelper::shadowRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void deepin_platform_plugin::DNoTitlebarWindowHelper::shadowOffectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void deepin_platform_plugin::DNoTitlebarWindowHelper::shadowColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void deepin_platform_plugin::DNoTitlebarWindowHelper::windowEffectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void deepin_platform_plugin::DNoTitlebarWindowHelper::windowStartUpEffectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void deepin_platform_plugin::DNoTitlebarWindowHelper::mouseInputAreaMarginsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
